// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_SERVANT_BASE_H
#define OB_SERVANT_BASE_H

namespace OB
{

//
// Forward declarations
//
class StubImplBase;

} // End of namespace OB


namespace PortableServer
{

//
// The PortableServer::ServantBase class, as defined by the C++ mapping
//
class ServantBase
{
#ifdef HAVE_JTC
    JTCRecursiveMutex servantSyncMutex_;
#endif

protected:
    
    ServantBase();
    
    ServantBase(const ServantBase&);
    
    ServantBase& operator=(const ServantBase&);
    
public:
    
    virtual ~ServantBase();
    
    virtual POA_ptr _default_POA();
    
    virtual CORBA::InterfaceDef_ptr _get_interface()
        throw(CORBA::SystemException);
        
    virtual CORBA::Boolean _is_a(const char* logical_type_id)
        throw(CORBA::SystemException);
        
    virtual CORBA::Boolean _non_existent()
        throw(CORBA::SystemException);
        
    virtual void _add_ref();
        
    virtual void _remove_ref();
    
    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    virtual OB::DispatchStatus _OB_dispatch(const char*, OB::Buffer&, bool,
					    CORBA::ULong, CORBA::ULong) = 0;
	
    OB::DispatchStatus _OB_dispatchBase(const char*, OB::Buffer&, bool,
					CORBA::ULong, CORBA::ULong);

    CORBA::Long _OB_findOperation(const char*, const char*[], CORBA::ULong);


    //
    // This method is so named so that we can use the same call as is
    // standard in PortableServer::DynamicImplementation
    //
    virtual char* //CORBA::RepositoryId
    _primary_interface(const ObjectId&, POA_ptr) = 0;

    CORBA::Object_ptr _OB_createReference();

    //
    // Is this a DSI servant?
    //
    virtual bool _OB_isDynamic();

    //
    // Create the OB::StubImplBase
    //
    virtual OB::StubImplBase* _OB_createStubImpl(POA_ptr,
                                                 const ObjectId&,
                                                 const OB::OctetSeq&) = 0;

    //
    // Lock and unlock per-servant mutex
    //
#ifdef HAVE_JTC
    void _OB_lock();
    void _OB_unlock();
#endif
};

//
// The PortableServer::RefCountServantBase class, as defined by the
// C++ mapping.
//
class RefCountServantBase : public virtual ServantBase,
                            public virtual OB::RefCount
{
protected:
    
    RefCountServantBase();

    RefCountServantBase(const RefCountServantBase&);
    
    RefCountServantBase& operator=(const RefCountServantBase&);

public:
    
    ~RefCountServantBase();
    
    virtual void _add_ref();
    virtual void _remove_ref();
};

} // End of namespace PortableServer

void OBRelease(PortableServer::ServantBase*);
void OBDuplicate(PortableServer::ServantBase*);

namespace PortableServer
{

typedef OB::ObjVar< ServantBase > ServantBase_var;

} // End of namespace PortableServer

#endif
